@@ -1,11 +1,11 @@
 ;; -*- no-byte-compile: t; lexical-binding: nil -*-
-(define-package "dashboard" "1.8.0"
+(define-package "dashboard" "20251231.1631"
   "A startup screen extracted from Spacemacs."
-  '((emacs "26.1"))
+  '((emacs "27.1"))
   :url "https://github.com/emacs-dashboard/emacs-dashboard"
-  :commit "a1c29c0bbfca3f6778022628b79e7eef2b9f351d"
-  :revdesc "a1c29c0bbfca"
+  :commit "49de4c16624f6a86a5beb1196f68f02636a74d1b"
+  :revdesc "49de4c16624f"
   :keywords '("startup" "screen" "tools" "dashboard")
   :authors '(("Rakan Al-Hneiti" . "rakan.alhneiti@gmail.com"))
-  :maintainers '(("Jesús Martínez" . "jesusmartinez93@gmail.com")
-                 ("Jen-Chieh" . "jcs090218@gmail.com")))
+  :maintainers '(("Jen-Chieh" . "jcs090218@gmail.com")
+                 ("Ricardo Arredondo" . "ricardo.richo@gmail.com")))
